/**
 * Copyright (c) 2000-2010 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.wsrp.messaging;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.messaging.Message;
import com.liferay.portal.kernel.messaging.MessageListener;
import com.liferay.wsrp.service.WSRPConsumerPortletLocalServiceUtil;

/**
 * <a href="WSRPMessageListener.java.html"><b><i>View Source</i></b></a>
 *
 * @author Hugo Huijser
 */
public class WSRPMessageListener implements MessageListener {

	public void receive(Message message){
		try {
			doReceive(message);
		}
		catch (Exception e) {
			_log.error("Unable to process message " + message, e);
		}
	}

	protected void doReceive(Message message) throws Exception {
		String command = message.getString("command");

		if (command.equals("add")) {
			long companyId = message.getLong("companyId");
			String name = message.getString("name");
			String portletHandle = message.getString("portletHandle");
			String userToken = message.getString("userToken");
			long wsrpConsumerId = message.getLong("wsrpConsumerId");
			long wsrpConsumerPortletId = message.getLong(
				"wsrpConsumerPortletId");

			WSRPConsumerPortletLocalServiceUtil.initWSRPConsumerPortlet(
				companyId, wsrpConsumerId, wsrpConsumerPortletId, name,
				portletHandle, userToken);
		}
		else if (command.equals("delete")) {
			String url = message.getString("url");
			long wsrpConsumerPortletId = message.getLong(
				"wsrpConsumerPortletId");

			WSRPConsumerPortletLocalServiceUtil.destroyWSRPConsumerPortlet(
				wsrpConsumerPortletId, url);
		}
	}

	private static Log _log = LogFactoryUtil.getLog(WSRPMessageListener.class);

}